/* Adaptation eines Programms von Andrew W. Appel (C) */
package straightparse;

%% 

%implements Lexer
%function nextToken
%type Symbol
%char

%{
private int stringstart;

private StringBuffer charbuf = null;

private void newline() {
  errorMsg.newline(yychar);
}

private void err(int pos, String s) {
  errorMsg.error(pos,s);
}

private void err(String s) {
  err(yychar,s);
}

private Symbol tok(int kind) {
    return new Symbol(kind, yychar, yychar+yylength());
}

private int comLevel = 0;
private errorMsg.ErrorMsg errorMsg;

Yylex(java.io.InputStream s, errorMsg.ErrorMsg e) {
  this(s);
  errorMsg=e;
}

%}

%eofval{
	{if (comLevel>0) err (stringstart, "unclosed comment");
	 return tok(Sym.EOF);
        }
%eofval}       


%state A S F
idchars=[A-Za-z_0-9]
id=[A-Za-z]{idchars}*
ws=("\012"|[\t\ ])*
%%
<YYINITIAL>{ws}	{}
<YYINITIAL>\n	{newline();}
<YYINITIAL>","	{return tok(Sym.COMMA);}
<YYINITIAL>";"	{return tok(Sym.SEMICOLON);}
<YYINITIAL>"("	{return tok(Sym.LPAREN);}
<YYINITIAL>")"	{return tok(Sym.RPAREN);}
<YYINITIAL>"+"	{return tok(Sym.PLUS);}
<YYINITIAL>"-"	{return tok(Sym.MINUS);}
<YYINITIAL>"*"	{return tok(Sym.TIMES);}
<YYINITIAL>"/"	{return tok(Sym.DIVIDE);}
<YYINITIAL>"="	{return tok(Sym.EQ);}
<YYINITIAL>"print" {return tok(Sym.PRINT);}
<YYINITIAL>{id}	{return new Symbol(Sym.ID,
				yychar,yychar+yylength(),
				     yytext());}
<YYINITIAL>[0-9]+ {return new Symbol(Sym.INT,
			yychar,yychar+yylength(),
			  new Integer(Integer.parseInt(yytext())));}
<YYINITIAL>"/*"	{ /* geloescht */}
<YYINITIAL>"*/"	{ /* geloescht */}
<YYINITIAL>.	{err("Falscher Buchstabe");}




